# Author: TheRealRazbi (https://github.com/TheRealRazbi)
# License: MPL-2.0
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.

__all__ = ['DGUSPicker', 'ManualPicker', 'TemplatePicker', 'OCRPicker']

from typing import Protocol

from stageclick.core import Window


class DGUSPicker(Protocol):
    def pick(self, window: Window, project_name: str) -> None:
        ...


class ManualPicker:
    def pick(self, *args, **kwargs):
        raise NotImplementedError


class TemplatePicker:
    def pick(self, *args, **kwargs):
        return NotImplementedError


class OCRPicker:
    def pick(self, *args, **kwargs):
        raise NotImplementedError



# Author: TheRealRazbi (https://github.com/TheRealRazbi)
# License: MPL-2.0
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.

import pytest
from unittest.mock import MagicMock, patch

from dgus_control import ExeStarter
from dgus_control.dgus import DGUS
from stageclick.core import Window

from dgus_control.exceptions import DGUSAlreadyStarted


def test_start_raises_if_dgus_already_running():
    with patch("stageclick.core.Window.find", return_value=MagicMock(spec=Window)):
        with pytest.raises(DGUSAlreadyStarted):
            dgus = DGUS(starter=ExeStarter("path/to/exe"))
            dgus.start()


def test_find_existing_dgus_window():
    mock_window = MagicMock(spec=Window)
    with patch("stageclick.core.Window.find", return_value=mock_window):
        dgus = DGUS(starter=ExeStarter('path/to/exe'))
        dgus.find()
        assert dgus.window == mock_window


def test_selects_window_when_not_visible():
    mock_window = MagicMock(spec=Window, visible=False)
    with patch.object(mock_window, "select") as mock_select:
        dgus = DGUS(starter=ExeStarter('path/to/exe'))
        dgus.window = mock_window
        dgus.select()
        mock_select.assert_called_once()


def test_skips_selection_if_window_is_visible():
    mock_window = MagicMock(spec=Window, visible=True)

    with patch("time.sleep") as mock_sleep, \
            patch.object(mock_window, "select") as mock_select:
        dgus = DGUS(starter=ExeStarter('path/to/exe'))
        dgus.window = mock_window
        dgus.select()
        mock_select.assert_called_once()
        mock_sleep.assert_not_called()



# Author: TheRealRazbi (https://github.com/TheRealRazbi)
# License: MPL-2.0
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.

__all__ = ['DGUSStarter', 'ExeStarter', 'ManualStarter']

from dataclasses import dataclass, field
from pathlib import Path
from typing import Protocol

from stageclick.core import Window, WindowNotFound
from stageclick.step_runner import StepRunner, is_done, grab_input_once, grab_y_n_bool
from termcolor import cprint

from dgus_control import DGUS_TITLE
from dgus_control.exceptions import DGUSAlreadyStarted


class DGUSStarter(Protocol):
    def start(self, *args, **kwargs) -> Window:
        ...

    def find(self, *args, **kwargs) -> Window:
        ...

    def find_or_start(self, *args, **kwargs) -> Window:
        ...


@dataclass
class ManualStarter:
    runner: StepRunner = field(default_factory=lambda: StepRunner('Manual Starter'))
    title: str = DGUS_TITLE

    @staticmethod
    def _find_dgus_by_exe_path(path):
        return Window.find_window_by_exe_path(path)

    def _input_function(self):
        def _is_valid(n):
            try:
                path = Path(n.strip('\'"'))
            except Exception as e:
                cprint(f"Error: {e}", "red")
                return
            if not path.exists():
                cprint(f"Exe path {path} doesn't exist", "red")
                return
            try:
                return self._find_dgus_by_exe_path(n)
            except WindowNotFound:
                cprint(f"DGUS not found by exe path {n}", "red")
                return

        def _process_result(n):
            window = self._find_dgus_by_exe_path(n.strip('\'"'))
            window.set_title(self.title)
            return window

        return grab_input_once(_is_valid, _process_result)

    def start(self, *args, **kwargs):
        self.runner.step("Please start the DGUS manually, 'done' when ready", is_done, skip_allowed=False)
        self.runner.step("Provide the path to the DGUS executable", self._input_function(), skip_allowed=False,
                         save_key="dgus_window")
        window: Window = self.runner.state['dgus_window']
        return window

    def find(self, *args, **kwargs):
        try:
            return Window.find(self.title)
        except WindowNotFound:
            if not kwargs.get('skip_start', False):
                self.runner.step(f"Window with title {self.title} not found, y for start(), n for quit()",
                                 grab_y_n_bool(), skip_allowed=False, save_key="start_confirm")
                if self.runner.state["start_confirm"]:
                    return self.start()
            raise

    def find_or_start(self, *args, **kwargs):
        try:
            return self.find(skip_start=True)
        except WindowNotFound:
            return self.start()


@dataclass
class ExeStarter:
    exe_path: str
    title: str = DGUS_TITLE
    wait_seconds: int = 6

    def start(self, *args, **kwargs):
        try:
            if Window.find(title=self.title):
                raise DGUSAlreadyStarted
        except WindowNotFound:
            ...
        window = Window.start_and_find(exe_path=self.exe_path, title=self.title, wait_seconds=self.wait_seconds)
        return window

    def find(self, *args, **kwargs):
        return Window.find(self.title)

    def find_or_start(self, *args, **kwargs):
        try:
            return self.find()
        except WindowNotFound:
            return self.start()



# Author: TheRealRazbi (https://github.com/TheRealRazbi)
# License: MPL-2.0
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.

from importlib.metadata import version, PackageNotFoundError
from .constants import *
from .dgus import *
from .dgus_starter import *
from .project_picker import *
from .exceptions import DGUSAlreadyStarted

try:
    __version__ = version("stage-click")
except PackageNotFoundError:
    __version__ = "unknown"

__all__ = [
    'version',
    *constants.__all__,
    *dgus.__all__,
    *dgus_starter.__all__,
    *project_picker.__all__,
]



# Author: TheRealRazbi (https://github.com/TheRealRazbi)
# License: MPL-2.0
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.

__all__ = ["DGUS"]

import time
from typing import Optional

from stageclick.core import Window

from dgus_control.dgus_starter import DGUSStarter
from dgus_control.project_picker import DGUSPicker


class DGUS:
    def __init__(self, starter: DGUSStarter, picker: DGUSPicker = None):
        self.starter = starter
        self.picker = picker
        self.window: Optional['Window'] = None

    def find_or_start(self):
        self.window = self.starter.find_or_start()

    def start(self):
        self.window = self.starter.start()

    def find(self):
        self.window = self.starter.find()

    def select(self):
        already_selected = self.window.visible
        self.window.select()
        if not already_selected:
            time.sleep(0.2)

    def is_running(self):
        return self.window is not None and self.window.is_running()
